@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2007-2012 Inverse inc.
+  Copyright (C) 2007-2014 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -45,6 +45,7 @@
 #import <NGImap4/NGImap4Client.h>
 #import <NGImap4/NGImap4Envelope.h>
 #import <NGImap4/NGImap4EnvelopeAddress.h>
+#import <NGMail/NGMailAddressParser.h>
 #import <NGMail/NGMimeMessage.h>
 #import <NGMail/NGMimeMessageGenerator.h>
 #import <NGMime/NGMimeBodyPart.h>
@@ -63,6 +64,11 @@
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserDefaults.h>
 
+#import <NGCards/NGVCard.h>
+
+#import <Contacts/SOGoContactFolders.h>
+#import <Contacts/SOGoContactGCSEntry.h>
+
 #import "NSData+Mail.h"
 #import "NSString+Mail.h"
 #import "SOGoMailAccount.h"
@@ -256,24 +262,11 @@ static NSString    *userAgent      = nil;
 
 /* contents */
 
-- (NSString *) _generateMessageID
-{
-  NSMutableString *messageID;
-  NSString *pGUID;
-
-  messageID = [NSMutableString string];
-  [messageID appendFormat: @"<%@", [self globallyUniqueObjectId]];
-  pGUID = [[NSProcessInfo processInfo] globallyUniqueString];
-  [messageID appendFormat: @"@%u>", [pGUID hash]];
-
-  return [messageID lowercaseString];
-}
-
 - (void) setHeaders: (NSDictionary *) newHeaders
 {
   id headerValue;
   unsigned int count;
-  NSString *messageID, *priority, *pureSender, *replyTo;
+  NSString *messageID, *priority, *pureSender, *replyTo, *receipt;
 
   for (count = 0; count < 8; count++)
     {
@@ -288,30 +281,61 @@ static NSString    *userAgent      = nil;
   messageID = [headers objectForKey: @"message-id"];
   if (!messageID)
     {
-      messageID = [self _generateMessageID];
+      messageID = [NSString generateMessageID];
       [headers setObject: messageID forKey: @"message-id"];
     }
   
-  priority = [newHeaders objectForKey: @"priority"];
-  if (!priority || [priority isEqualToString: @"NORMAL"])
+  priority = [newHeaders objectForKey: @"X-Priority"];
+  if (priority)
     {
-      [headers removeObjectForKey: @"X-Priority"];
-    }
-  else if ([priority isEqualToString: @"HIGHEST"])
-    {
-      [headers setObject: @"1 (Highest)"  forKey: @"X-Priority"];
-    }
-  else if ([priority isEqualToString: @"HIGH"])
-    {
-      [headers setObject: @"2 (High)"  forKey: @"X-Priority"];
-    }
-  else if ([priority isEqualToString: @"LOW"])
-    {
-      [headers setObject: @"4 (Low)"  forKey: @"X-Priority"];
+      // newHeaders come from MIME message; convert X-Priority to Web representation
+      [headers setObject: priority  forKey: @"X-Priority"];
+      [headers removeObjectForKey: @"priority"];
+      if ([priority isEqualToString: @"1 (Highest)"])
+        {
+          [headers setObject: @"HIGHEST"  forKey: @"priority"];
+        }
+      else if ([priority isEqualToString: @"2 (High)"])
+        {
+          [headers setObject: @"HIGH"  forKey: @"priority"];
+        }
+      else if ([priority isEqualToString: @"4 (Low)"])
+        {
+          [headers setObject: @"LOW"  forKey: @"priority"];
+        }
+      else if ([priority isEqualToString: @"5 (Lowest)"])
+        {
+          [headers setObject: @"LOWEST"  forKey: @"priority"];
+        }
     }
   else
     {
-      [headers setObject: @"5 (Lowest)"  forKey: @"X-Priority"];
+      // newHeaders come from Web form; convert priority to MIME header representation
+      priority = [newHeaders objectForKey: @"priority"];
+      if (!priority || [priority isEqualToString: @"NORMAL"])
+        {
+          [headers removeObjectForKey: @"X-Priority"];
+        }
+      else if ([priority isEqualToString: @"HIGHEST"])
+        {
+          [headers setObject: @"1 (Highest)"  forKey: @"X-Priority"];
+        }
+      else if ([priority isEqualToString: @"HIGH"])
+        {
+          [headers setObject: @"2 (High)"  forKey: @"X-Priority"];
+        }
+      else if ([priority isEqualToString: @"LOW"])
+        {
+          [headers setObject: @"4 (Low)"  forKey: @"X-Priority"];
+        }
+      else
+        {
+          [headers setObject: @"5 (Lowest)"  forKey: @"X-Priority"];
+        }
+      if (priority)
+        {
+          [headers setObject: priority  forKey: @"priority"];
+        }
     }
 
   replyTo = [headers objectForKey: @"replyTo"];
@@ -321,14 +345,30 @@ static NSString    *userAgent      = nil;
     }
   [headers removeObjectForKey: @"replyTo"];
 
-  if ([[newHeaders objectForKey: @"receipt"] isEqualToString: @"true"])
+  receipt = [newHeaders objectForKey: @"Disposition-Notification-To"];
+  if ([receipt length] > 0)
     {
-      pureSender = [[newHeaders objectForKey: @"from"] pureEMailAddress];
-      if (pureSender)
-        [headers setObject: pureSender forKey: @"Disposition-Notification-To"];
+      [headers setObject: @"true"  forKey: @"receipt"];
+      [headers setObject: receipt forKey: @"Disposition-Notification-To"];
     }
   else
-    [headers removeObjectForKey: @"Disposition-Notification-To"];
+    {
+      receipt = [newHeaders objectForKey: @"receipt"];
+      if ([receipt isEqualToString: @"true"])
+        {
+          [headers setObject: receipt  forKey: @"receipt"];
+          pureSender = [[newHeaders objectForKey: @"from"] pureEMailAddress];
+          if (pureSender)
+            {
+              [headers setObject: pureSender forKey: @"Disposition-Notification-To"];
+            }
+        }
+      else
+        {
+          [headers removeObjectForKey: @"receipt"];
+          [headers removeObjectForKey: @"Disposition-Notification-To"];
+        }
+    }
 }
 
 - (NSDictionary *) headers
@@ -356,6 +396,11 @@ static NSString    *userAgent      = nil;
   return isHTML;
 }
 
+- (NSString *) inReplyTo
+{
+  return inReplyTo;
+}
+
 - (void) setInReplyTo: (NSString *) newInReplyTo
 {
   ASSIGN (inReplyTo, newInReplyTo);
@@ -395,11 +440,17 @@ static NSString    *userAgent      = nil;
   [self setSourceFolder: [paths componentsJoinedByString: @"/"]];
 }
 
+//
+//
+//
 - (NSString *) sourceFolder
 {
   return sourceFolder;
 }
 
+//
+// Store the message definition in a plist file (.info.plist) in the spool directory
+//
 - (NSException *) storeInfo
 {
   NSMutableDictionary *infos;
@@ -446,6 +497,9 @@ static NSString    *userAgent      = nil;
   return error;
 }
 
+//
+//
+//
 - (void) _loadInfosFromDictionary: (NSDictionary *) infoDict
 {
   id value;
@@ -478,11 +532,17 @@ static NSString    *userAgent      = nil;
     [self setInReplyTo: value];
 }
 
+//
+//
+//
 - (NSString *) relativeImap4Name
 {
   return [NSString stringWithFormat: @"%d", IMAP4ID];
 }
 
+//
+//
+//
 - (void) fetchInfo
 {
   NSString *p;
@@ -504,16 +564,25 @@ static NSString    *userAgent      = nil;
     [self debugWithFormat: @"Note: info object does not yet exist: %@", p];
 }
 
+//
+//
+//
 - (void) setIMAP4ID: (int) newIMAP4ID
 {
   IMAP4ID = newIMAP4ID;
 }
 
+//
+//
+//
 - (int) IMAP4ID
 {
   return IMAP4ID;
 }
 
+//
+//
+//
 - (NSException *) save
 {
   NGImap4Client *client;
@@ -535,23 +604,31 @@ static NSString    *userAgent      = nil;
     }
   
   folder = [imap4 imap4FolderNameForURL: [container imap4URL]];
-  result
-    = [client append: message toFolder: folder
-	      withFlags: [NSArray arrayWithObjects: @"seen", @"draft", nil]];
+  result = [client append: message toFolder: folder
+                withFlags: [NSArray arrayWithObjects: @"seen", @"draft", nil]];
   if ([[result objectForKey: @"result"] boolValue])
     {
       if (IMAP4ID > -1)
 	error = [imap4 markURLDeleted: [self imap4URL]];
       IMAP4ID = [self IMAP4IDFromAppendResult: result];
+      if (imap4URL)
+        {
+          // Invalidate the IMAP message URL since the message ID has changed
+          [imap4URL release];
+          imap4URL = nil;
+        }
       [self storeInfo];
     }
   else
-    error = [NSException exceptionWithHTTPStatus:500 /* Server Error */
-			 reason: @"Failed to store message"];
+    error = [NSException exceptionWithHTTPStatus: 500 /* Server Error */
+                                          reason: [result objectForKey: @"reason"]];
 
   return error;
 }
 
+//
+//
+//
 - (void) _addEMailsOfAddresses: (NSArray *) _addrs
 		       toArray: (NSMutableArray *) _ma
 {
@@ -564,6 +641,9 @@ static NSString    *userAgent      = nil;
       [_ma addObject: [currentAddress email]];
 }
 
+//
+//
+//
 - (void) _addRecipients: (NSArray *) recipients
 	        toArray: (NSMutableArray *) array
 {
@@ -576,6 +656,9 @@ static NSString    *userAgent      = nil;
       [array addObject: [currentAddress baseEMail]];
 }
 
+//
+//
+//
 - (void) _purgeRecipients: (NSArray *) recipients
 	    fromAddresses: (NSMutableArray *) addresses
 {
@@ -602,6 +685,9 @@ static NSString    *userAgent      = nil;
       }
 }
 
+//
+//
+//
 - (void) _fillInReplyAddresses: (NSMutableDictionary *) _info
 		    replyToAll: (BOOL) _replyToAll
 		      envelope: (NGImap4Envelope *) _envelope
@@ -711,6 +797,9 @@ static NSString    *userAgent      = nil;
     }
 }
 
+//
+//
+//
 - (NSArray *) _attachmentBodiesFromPaths: (NSArray *) paths
 		       fromResponseFetch: (NSDictionary *) fetch;
 {
@@ -731,15 +820,18 @@ static NSString    *userAgent      = nil;
   return bodies;
 }
 
-- (void) _fetchAttachments: (NSArray *) parts
-                  fromMail: (SOGoMailObject *) sourceMail
+//
+//
+//
+- (void) _fetchAttachmentsFromMail: (SOGoMailObject *) sourceMail
 {
   unsigned int count, max;
-  NSArray *paths, *bodies;
+  NSArray *parts, *paths, *bodies;
   NSData *body;
   NSDictionary *currentInfo;
   NGHashMap *response;
 
+  parts = [sourceMail fetchFileAttachmentKeys];
   max = [parts count];
   if (max > 0)
     {
@@ -758,17 +850,21 @@ static NSString    *userAgent      = nil;
     }
 }
 
+//
+//
+//
 - (void) fetchMailForEditing: (SOGoMailObject *) sourceMail
 {
   NSString *subject, *msgid;
   NSMutableDictionary *info;
+  NSDictionary *h;
   NSMutableArray *addresses;
   NGImap4Envelope *sourceEnvelope;
+  id priority, receipt;
 
   [sourceMail fetchCoreInfos];
 
-  [self _fetchAttachments: [sourceMail fetchFileAttachmentKeys]
-	fromMail: sourceMail];
+  [self _fetchAttachmentsFromMail: sourceMail];
   info = [NSMutableDictionary dictionaryWithCapacity: 16];
   subject = [sourceMail subject];
   if ([subject length] > 0)
@@ -794,6 +890,15 @@ static NSString    *userAgent      = nil;
   [self _addEMailsOfAddresses: [sourceEnvelope replyTo] toArray: addresses];
   if ([addresses count] > 0)
     [info setObject: addresses forKey: @"replyTo"];
+
+  h = [sourceMail mailHeaders];
+  priority = [h objectForKey: @"x-priority"];
+  if ([priority isNotEmpty] && [priority isKindOfClass: [NSString class]])
+      [info setObject: (NSString*)priority forKey: @"X-Priority"];
+  receipt = [h objectForKey: @"disposition-notification-to"];
+  if ([receipt isNotEmpty] && [receipt isKindOfClass: [NSString class]])
+      [info setObject: (NSString*)receipt forKey: @"Disposition-Notification-To"];
+
   [self setHeaders: info];
 
   [self setText: [sourceMail contentForEditing]];
@@ -804,10 +909,13 @@ static NSString    *userAgent      = nil;
   [self storeInfo];
 }
 
+//
+//
+//
 - (void) fetchMailForReplying: (SOGoMailObject *) sourceMail
 			toAll: (BOOL) toAll
 {
-  NSString *contentForReply, *msgID;
+  NSString *msgID;
   NSMutableDictionary *info;
   NGImap4Envelope *sourceEnvelope;
 
@@ -818,12 +926,11 @@ static NSString    *userAgent      = nil;
 
   sourceEnvelope = [sourceMail envelope];
   [self _fillInReplyAddresses: info replyToAll: toAll
-	envelope: sourceEnvelope];
+                     envelope: sourceEnvelope];
   msgID = [sourceEnvelope messageID];
   if ([msgID length] > 0)
     [self setInReplyTo: msgID];
-  contentForReply = [sourceMail contentForReply];
-  [self setText: contentForReply];
+  [self setText: [sourceMail contentForReply]];
   [self setHeaders: info];
   [self setSourceURL: [sourceMail imap4URLString]];
   [self setSourceFlag: @"Answered"];
@@ -858,8 +965,7 @@ static NSString    *userAgent      = nil;
   if ([[ud mailMessageForwarding] isEqualToString: @"inline"])
     {
       [self setText: [sourceMail contentForInlineForward]];
-      [self _fetchAttachments: [sourceMail fetchFileAttachmentKeys]
-	    fromMail: sourceMail];
+      [self _fetchAttachmentsFromMail: sourceMail];
     }
   else
     {
@@ -869,17 +975,21 @@ static NSString    *userAgent      = nil;
       if ([signature length])
         {
           nl = (isHTML ? @"<br/>" : @"\n");
-          [self setText: [NSString stringWithFormat: @"%@-- %@%@", nl, nl, signature]];
+          [self setText: [NSString stringWithFormat: @"%@%@-- %@%@", nl, nl, nl, signature]];
         }
       attachment = [NSDictionary dictionaryWithObjectsAndKeys:
 				   [sourceMail filenameForForward], @"filename",
 				 @"message/rfc822", @"mimetype",
 				 nil];
       [self saveAttachment: [sourceMail content]
-	    withMetadata: attachment];
+              withMetadata: attachment];
     }
 
   [self storeInfo];
+
+  // Save the message to the IMAP store so the user can eventually view the attached file(s)
+  // from the Web interface
+  [self save];
 }
 
 /* accessors */
@@ -898,13 +1008,19 @@ static NSString    *userAgent      = nil;
 
 /* attachments */
 
-- (NSArray *) fetchAttachmentNames
+//
+// Return the attributes (name, size and mime body part) of the files found in the draft folder
+// on the local filesystem
+//
+- (NSArray *) fetchAttachmentAttrs
 {
   NSMutableArray *ma;
   NSFileManager *fm;
   NSArray *files;
-  unsigned count, max;
   NSString *filename;
+  NSDictionary *fileAttrs;
+  NGMimeBodyPart *bodyPart;
+  unsigned count, max;
 
   fm = [NSFileManager defaultManager];
   files = [fm directoryContentsAtPath: [self draftFolderPath]];
@@ -915,7 +1031,13 @@ static NSString    *userAgent      = nil;
     {
       filename = [files objectAtIndex: count];
       if (![filename hasPrefix: @"."])
-	[ma addObject: filename];
+        {
+          fileAttrs = [fm fileAttributesAtPath: [self pathToAttachmentWithName: filename] traverseLink: YES];
+          bodyPart = [self bodyPartForAttachmentWithName: filename];
+          [ma addObject: [NSDictionary dictionaryWithObjectsAndKeys: filename, @"filename",
+                                       [fileAttrs objectForKey: @"NSFileSize"], @"size",
+                                       bodyPart, @"part", nil]];
+        }
     }
 
   return ma;
@@ -1174,8 +1296,8 @@ static NSString    *userAgent      = nil;
   /* check attachment */
   
   fm = [NSFileManager defaultManager];
-  p  = [self pathToAttachmentWithName:_name];
-  if (![fm isReadableFileAtPath:p]) {
+  p  = [self pathToAttachmentWithName: _name];
+  if (![fm isReadableFileAtPath: p]) {
     [self errorWithFormat: @"did not find attachment: '%@'", _name];
     return nil;
   }
@@ -1184,21 +1306,21 @@ static NSString    *userAgent      = nil;
   
   /* prepare header of body part */
 
-  map = [[[NGMutableHashMap alloc] initWithCapacity:4] autorelease];
+  map = [[[NGMutableHashMap alloc] initWithCapacity: 4] autorelease];
 
   if ((s = [self contentTypeForAttachmentWithName:_name]) != nil) {
-    [map setObject:s forKey: @"content-type"];
+    [map setObject: s forKey: @"content-type"];
     if ([s hasPrefix: @"text/plain"] || [s hasPrefix: @"text/html"])
       attachAsString = YES;
     else if ([s hasPrefix: @"message/rfc822"])
       attachAsRFC822 = YES;
   }
-  if ((s = [self contentDispositionForAttachmentWithName:_name]))
+  if ((s = [self contentDispositionForAttachmentWithName: _name]))
     {
       NGMimeContentDispositionHeaderField *o;
       
       o = [[NGMimeContentDispositionHeaderField alloc] initWithString: s];
-      [map setObject:o forKey: @"content-disposition"];
+      [map setObject: o forKey: @"content-disposition"];
       [o release];
     }
   
@@ -1233,7 +1355,6 @@ static NSString    *userAgent      = nil;
     if (attachAsRFC822)
       {
         [map setObject: @"8bit" forKey: @"content-transfer-encoding"];
-        [map setObject: @"inline" forKey: @"content-disposition"];
       }
     else
       {
@@ -1255,27 +1376,33 @@ static NSString    *userAgent      = nil;
   return bodyPart;
 }
 
+//
+//
+//
 - (NSArray *) bodyPartsForAllAttachments
 {
   /* returns nil on error */
-  NSArray  *names;
+  NSArray  *attrs;
   unsigned i, count;
   NGMimeBodyPart *bodyPart;
   NSMutableArray *bodyParts;
 
-  names = [self fetchAttachmentNames];
-  count = [names count];
+  attrs = [self fetchAttachmentAttrs];
+  count = [attrs count];
   bodyParts = [NSMutableArray arrayWithCapacity: count];
 
   for (i = 0; i < count; i++)
     {
-      bodyPart = [self bodyPartForAttachmentWithName: [names objectAtIndex: i]];
+      bodyPart = [self bodyPartForAttachmentWithName: [[attrs objectAtIndex: i] objectForKey: @"filename"]];
       [bodyParts addObject: bodyPart];
     }
 
   return bodyParts;
 }
 
+//
+//
+//
 - (NGMimeBodyPart *) mimeMultipartAlternative
 {
   NGMimeMultipartBody *textParts;
@@ -1301,6 +1428,9 @@ static NSString    *userAgent      = nil;
   return part;
 }
 
+//
+//
+//
 - (NGMimeMessage *) mimeMultiPartMessageWithHeaderMap: (NGMutableHashMap *) map
 					 andBodyParts: (NSArray *) _bodyParts
 {
@@ -1340,6 +1470,9 @@ static NSString    *userAgent      = nil;
   return message;
 }
 
+//
+//
+//
 - (void) _addHeaders: (NSDictionary *) _h
          toHeaderMap: (NGMutableHashMap *) _map
 {
@@ -1474,10 +1607,11 @@ static NSString    *userAgent      = nil;
   /* add subject */
   if ([(s = [headers objectForKey: @"subject"]) length] > 0)
     [map setObject: [s asQPSubjectString: @"utf-8"]
-	 forKey: @"subject"];
-
-  [map setObject: [headers objectForKey: @"message-id"]
-       forKey: @"message-id"];
+            forKey: @"subject"];
+  
+  if ([(s = [headers objectForKey: @"message-id"]) length] > 0)
+    [map setObject: s
+            forKey: @"message-id"];
 
   /* add standard headers */
   dateString = [[NSCalendarDate date] rfc822DateString];
@@ -1511,60 +1645,78 @@ static NSString    *userAgent      = nil;
   return map;
 }
 
+//
+//
+//
 - (NGMimeMessage *) mimeMessageWithHeaders: (NSDictionary *) _headers
 				 excluding: (NSArray *) _exclude
+                          extractingImages: (BOOL) _extractImages
 {
-  NGMutableHashMap  *map;
-  NSArray           *bodyParts;
-  NGMimeMessage     *message;
+  NSMutableArray *bodyParts;
+  NGMimeMessage *message;
+  NGMutableHashMap *map;
+  NSString *newText;
 
   message = nil;
 
+  bodyParts = [NSMutableArray array];
+
+  if (_extractImages)
+    {
+      newText = [text htmlByExtractingImages: bodyParts];
+      if ([bodyParts count])
+        [self setText: newText];
+    }
+
   map = [self mimeHeaderMapWithHeaders: _headers
-	      excluding: _exclude];
+                             excluding: _exclude];
   if (map)
     {
       //[self debugWithFormat: @"MIME Envelope: %@", map];
-  
-      bodyParts = [self bodyPartsForAllAttachments];
-      if (bodyParts)
-	{
-	  //[self debugWithFormat: @"attachments: %@", bodyParts];
-  
-	  if ([bodyParts count] == 0)
-	    /* no attachments */
-	    message = [self mimeMessageForContentWithHeaderMap: map];
-	  else
-	    /* attachments, create multipart/mixed */
-	    message = [self mimeMultiPartMessageWithHeaderMap: map 
-			    andBodyParts: bodyParts];
-	  //[self debugWithFormat: @"message: %@", message];
-	}
+      
+      [bodyParts addObjectsFromArray: [self bodyPartsForAllAttachments]];
+      
+      //[self debugWithFormat: @"attachments: %@", bodyParts];
+      
+      if ([bodyParts count] == 0)
+        /* no attachments */
+        message = [self mimeMessageForContentWithHeaderMap: map];
       else
-	[self errorWithFormat:
-		@"could not create body parts for attachments!"];
+        /* attachments, create multipart/mixed */
+        message = [self mimeMultiPartMessageWithHeaderMap: map 
+                                             andBodyParts: bodyParts];
+      //[self debugWithFormat: @"message: %@", message];
     }
-
+  
   return message;
 }
 
+//
+// Return a NGMimeMessage object with inline HTML images (<img src=data>) extracted as attachments (<img src=cid>).
+//
 - (NGMimeMessage *) mimeMessage
 {
-  return [self mimeMessageWithHeaders: nil  excluding: nil];
+  return [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: YES];
 }
 
+//
+// Return a NSData object of the message with no alteration.
+//
 - (NSData *) mimeMessageAsData
 {
   NGMimeMessageGenerator *generator;
   NSData *message;
 
   generator = [NGMimeMessageGenerator new];
-  message = [generator generateMimeFromPart: [self mimeMessage]];
+  message = [generator generateMimeFromPart: [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: NO]];
   [generator release];
 
   return message;
 }
 
+//
+//
+//
 - (NSArray *) allRecipients
 {
   NSMutableArray *allRecipients;
@@ -1578,12 +1730,15 @@ static NSString    *userAgent      = nil;
     {
       recipients = [headers objectForKey: fieldNames[count]];
       if ([recipients count] > 0)
-	[allRecipients addObjectsFromArray: recipients];
+        [allRecipients addObjectsFromArray: recipients];
     }
 
   return allRecipients;
 }
 
+//
+//
+//
 - (NSArray *) allBareRecipients
 {
   NSMutableArray *bareRecipients;
@@ -1599,11 +1754,74 @@ static NSString    *userAgent      = nil;
   return bareRecipients;
 }
 
+//
+//
+//
 - (NSException *) sendMail
 {
+  SOGoUserDefaults *ud;
+  ud = [[context activeUser] userDefaults];
+
+  if ([ud mailAddOutgoingAddresses])
+  {
+    SOGoContactFolders *contactFolders;
+    NGMailAddressParser *parser;
+    id parsedRecipient;
+    SOGoContactFolder *folder;
+    SOGoContactGCSEntry *newContact;
+    NGVCard *card;
+    Class contactGCSEntry;
+    NSMutableArray *recipients;
+    NSString *recipient, *emailAddress, *addressBook, *uid;
+    NSArray *matchingContacts;
+    int i;
+
+    // Get all the addressbooks
+    contactFolders = [[[context activeUser] homeFolderInContext: context]
+                                                     lookupName: @"Contacts"
+                                                      inContext: context
+                                                      acquire: NO];
+    // Get all the recipients from the current email
+    recipients = [self allRecipients];
+    for (i = 0; i < [recipients count]; i++)
+    {
+      // The address contains a string. ex: "John Doe <sogo1@exemple.com>"
+      recipient = [recipients objectAtIndex: i];
+      parser = [NGMailAddressParser mailAddressParserWithString: recipient];
+      parsedRecipient = [parser parse];
+      emailAddress = [parsedRecipient address];
+
+      matchingContacts = [contactFolders allContactsFromFilter: emailAddress
+                                                 excludeGroups: YES
+                                                  excludeLists: YES];
+    }
+    // If we don't get any results from the autocompletion code, we add it..
+    if ([matchingContacts count] == 0)
+    {
+      // Get the selected addressbook from the user preferences where the new address will be added
+      addressBook = [ud selectedAddressBook];
+      folder = [contactFolders lookupName: addressBook inContext: context  acquire: NO];
+      uid = [folder globallyUniqueObjectId];
+      
+      if (folder && uid)
+      {
+        card = [NGVCard cardWithUid: uid];
+        [card addEmail: emailAddress types: nil];
+        
+        contactGCSEntry = NSClassFromString(@"SOGoContactGCSEntry");
+        newContact = [contactGCSEntry objectWithName: uid
+                                         inContainer: folder];
+        [newContact setIsNew: YES];
+        [newContact saveContentString: [card versitString]];
+      }
+    }
+  }
   return [self sendMailAndCopyToSent: YES];
 }
 
+//
+//
+//
 - (NSException *) sendMailAndCopyToSent: (BOOL) copyToSent
 {
   NSMutableData *cleaned_message;
@@ -1620,9 +1838,8 @@ static NSString    *userAgent      = nil;
   NGMimeMessageGenerator *generator;
 
   generator = [[[NGMimeMessageGenerator alloc] init] autorelease];
-  message = [generator generateMimeFromPart: [self mimeMessageWithHeaders: nil 
-                                                                excluding: nil]];
-      
+  message = [generator generateMimeFromPart: [self mimeMessage]];
+
   //
   // We now look for the Bcc: header. If it is present, we remove it.
   // Some servers, like qmail, do not remove it automatically.
