@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2007-2009 Inverse inc.
+  Copyright (C) 2007-2014 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -52,6 +52,7 @@
 
 #import "NSString+Mail.h"
 #import "NSData+Mail.h"
+#import "NSDictionary+Mail.h"
 #import "SOGoMailFolder.h"
 #import "SOGoMailAccount.h"
 #import "SOGoMailAccounts.h"
@@ -433,7 +434,8 @@ static BOOL debugSoParts       = NO;
   NSData *content;
   id     result, fullResult;
   
-  fullResult = [self fetchParts: [NSArray arrayWithObject: @"RFC822"]];
+  // We avoid using RFC822 here as the part name as it'll flag the message as Seen
+  fullResult = [self fetchParts: [NSArray arrayWithObject: @"BODY.PEEK[]"]];
   if (fullResult == nil)
     return nil;
   
@@ -457,7 +459,7 @@ static BOOL debugSoParts       = NO;
   
   /* extract message */
   
-  if ((content = [result valueForKey: @"message"]) == nil) {
+  if ((content = [[result valueForKey: @"body[]"] valueForKey: @"data"]) == nil) {
     [self logWithFormat:
 	    @"ERROR: unexpected IMAP4 result (missing 'message'): %@", 
 	    result];
@@ -503,26 +505,6 @@ static BOOL debugSoParts       = NO;
 }
 
 /* bulk fetching of plain/text content */
-
-// - (BOOL) shouldFetchPartOfType: (NSString *) _type
-// 		       subtype: (NSString *) _subtype
-// {
-//   /*
-//     This method decides which parts are 'prefetched' for display. Those are
-//     usually text parts (the set is currently hardcoded in this method ...).
-//   */
-//   _type    = [_type    lowercaseString];
-//   _subtype = [_subtype lowercaseString];
-  
-//   return (([_type isEqualToString: @"text"]
-//            && ([_subtype isEqualToString: @"plain"]
-//                || [_subtype isEqualToString: @"html"]
-//                || [_subtype isEqualToString: @"calendar"]))
-//           || ([_type isEqualToString: @"application"]
-//               && ([_subtype isEqualToString: @"pgp-signature"]
-//                   || [_subtype hasPrefix: @"x-vnd.kolab."])));
-// }
-
 - (void) addRequiredKeysOfStructure: (NSDictionary *) info
 			       path: (NSString *) p
 			    toArray: (NSMutableArray *) keys
@@ -622,9 +604,12 @@ static BOOL debugSoParts       = NO;
 		   @"text/calendar", @"application/ics",
 		   @"application/pgp-signature", nil];
   ma = [NSMutableArray arrayWithCapacity: 4];
+
   [self addRequiredKeysOfStructure: [self bodyStructure]
-	path: @"" toArray: ma acceptedTypes: types
-        withPeek: NO];
+                              path: @""
+                           toArray: ma 
+                     acceptedTypes: types
+                          withPeek: YES];
 
   return ma;
 }
@@ -654,6 +639,12 @@ static BOOL debugSoParts       = NO;
     NSData   *data;
     
     key  = [[_fetchKeys objectAtIndex:i] objectForKey: @"key"];
+
+    // We'll ask for the body.peek[] but SOPE returns us body[] responses
+    // so the key won't ever be found.
+    if ([key hasPrefix: @"body.peek["])
+      key = [NSString stringWithFormat: @"body[%@", [key substringFromIndex: 10]];
+
     data = [(NSDictionary *)[(NSDictionary *)result objectForKey:key] 
 			    objectForKey: @"data"];
     
@@ -705,9 +696,9 @@ static BOOL debugSoParts       = NO;
   return urlToPart;
 }
 
-- (void) _feedAttachmentIds: (NSMutableDictionary *) attachmentIds
-		  withInfos: (NSDictionary *) infos
-		  andPrefix: (NSString *) prefix
+- (void) _feedFileAttachmentIds: (NSMutableDictionary *) attachmentIds
+                      withInfos: (NSDictionary *) infos
+                      andPrefix: (NSString *) prefix
 {
   NSArray *parts;
   NSDictionary *currentPart;
@@ -727,14 +718,14 @@ static BOOL debugSoParts       = NO;
   for (count = 0; count < max; count++)
     {
       currentPart = [parts objectAtIndex: count];
-      [self _feedAttachmentIds: attachmentIds
-	    withInfos: currentPart
-	    andPrefix: [NSString stringWithFormat: @"%@/%d",
-				 prefix, count + 1]];
+      [self _feedFileAttachmentIds: attachmentIds
+                         withInfos: currentPart
+                         andPrefix: [NSString stringWithFormat: @"%@/%d",
+                                              prefix, count + 1]];
     }
 }
 
-- (NSDictionary *) fetchAttachmentIds
+- (NSDictionary *) fetchFileAttachmentIds
 {
   NSMutableDictionary *attachmentIds;
   NSString *prefix;
@@ -745,13 +736,125 @@ static BOOL debugSoParts       = NO;
   prefix = [[self soURL] absoluteString];
   if ([prefix hasSuffix: @"/"])
     prefix = [prefix substringToIndex: [prefix length] - 1];
-  [self _feedAttachmentIds: attachmentIds
+  [self _feedFileAttachmentIds: attachmentIds
 	withInfos: [coreInfos objectForKey: @"bodystructure"]
 	andPrefix: prefix];
 
   return attachmentIds;
 }
 
+//
+//
+//
+- (void) _fetchFileAttachmentKey: (NSDictionary *) part
+		       intoArray: (NSMutableArray *) keys
+		        withPath: (NSString *) path
+                       andPrefix: (NSString *) prefix
+{
+  NSString *filename, *mimeType, *filenameURL;
+  NSDictionary *currentFile;
+
+  filename = [part filename];
+
+  mimeType = [NSString stringWithFormat: @"%@/%@",
+		       [part objectForKey: @"type"],
+		       [part objectForKey: @"subtype"]];
+
+  if (!filename)
+      // We might end up here because of MUA that actually strips the
+      // Content-Disposition (and thus, the filename) when mails containing
+      // attachments have been forwarded. Thunderbird (2.x) does just that
+      // when forwarding mails with images attached to them (using cid:...).
+      if ([mimeType hasPrefix: @"application/"] ||
+	  [mimeType hasPrefix: @"audio/"] ||
+	  [mimeType hasPrefix: @"image/"] ||
+	  [mimeType hasPrefix: @"video/"])
+          filename = [NSString stringWithFormat: @"unknown_%@", path];
+      else if ([mimeType isEqualToString: @"message/rfc822"])
+        filename = [NSString stringWithFormat: @"email_%@.eml", path];
+  
+
+  if (filename)
+    {
+      // We replace any slash by a dash since Apache won't allow encoded slashes by default.
+      // See http://httpd.apache.org/docs/2.2/mod/core.html#allowencodedslashes
+      // See [UIxMailPartViewer _filenameForAttachment:]
+      filenameURL = [[filename stringByReplacingString: @"/" withString: @"-"] stringByEscapingURL];
+      currentFile = [NSDictionary dictionaryWithObjectsAndKeys:
+                                  filename, @"filename",
+                                  [mimeType lowercaseString], @"mimetype",
+                                  path, @"path",
+                                  [part objectForKey: @"encoding"], @"encoding",
+                                  [part objectForKey:@ "size"], @"size",
+                                  [NSString stringWithFormat: @"%@/%@", prefix, filenameURL], @"url",
+                                  [NSString stringWithFormat: @"%@/asAttachment/%@", prefix, filenameURL], @"urlAsAttachment",
+                                  nil];
+      [keys addObject: currentFile];
+    }
+}
+
+//
+//
+//
+- (void) _fetchFileAttachmentKeysInPart: (NSDictionary *) part
+                              intoArray: (NSMutableArray *) keys
+                               withPath: (NSString *) path
+                              andPrefix: (NSString *) prefix
+{
+  NSMutableDictionary *currentPart;
+  NSString *newPath;
+  NSArray *subparts;
+  NSString *type;
+  NSUInteger i;
+
+  type = [[part objectForKey: @"type"] lowercaseString];
+  if ([type isEqualToString: @"multipart"])
+    {
+      subparts = [part objectForKey: @"parts"];
+      for (i = 1; i <= [subparts count]; i++)
+	{
+	  currentPart = [subparts objectAtIndex: i-1];
+	  if (path)
+	    newPath = [NSString stringWithFormat: @"%@.%d", path, i];
+	  else
+	    newPath = [NSString stringWithFormat: @"%d", i];
+	  [self _fetchFileAttachmentKeysInPart: currentPart
+                                     intoArray: keys
+                                      withPath: newPath
+                                     andPrefix: [NSString stringWithFormat: @"%@/%i", prefix, i]];
+	}
+    }
+  else
+    {
+      if (!path)
+        path = @"1";
+      [self _fetchFileAttachmentKey: part
+                          intoArray: keys
+                           withPath: path
+                          andPrefix: prefix];
+    }
+}
+
+//
+//
+//
+#warning we might need to handle parts with a "name" attribute
+- (NSArray *) fetchFileAttachmentKeys
+{
+  NSString *prefix;
+  NSMutableArray *keys;
+
+  prefix = [[self soURL] absoluteString];
+  if ([prefix hasSuffix: @"/"])
+    prefix = [prefix substringToIndex: [prefix length] - 1];
+
+  keys = [NSMutableArray array];
+  [self _fetchFileAttachmentKeysInPart: [self bodyStructure]
+                             intoArray: keys withPath: nil andPrefix: prefix];
+
+  return keys;
+}
+
 /* convert parts to strings */
 - (NSString *) stringForData: (NSData *) _data
 		    partInfo: (NSDictionary *) _info
@@ -1235,11 +1338,6 @@ static BOOL debugSoParts       = NO;
   return mailETag;
 }
 
-- (int) zlGenerationCount
-{
-  return 0; /* mails never change */
-}
-
 - (NSArray *) aclsForUser: (NSString *) uid
 {
   return [container aclsForUser: uid];
@@ -1353,7 +1451,7 @@ static BOOL debugSoParts       = NO;
 
 - (BOOL) hasAttachment
 {
-  return ([[self fetchAttachmentIds] count] > 0);
+  return ([[self fetchFileAttachmentKeys] count] > 0);
 }
 
 - (BOOL) isNewMail
@@ -1423,7 +1521,7 @@ static BOOL debugSoParts       = NO;
   NSRange range;
 
   rc = nil;
-  fetch = [self _fetchProperty: @"BODY[HEADER.FIELDS (RECEIVED)]"];
+  fetch = [self _fetchProperty: @"BODY.PEEK[HEADER.FIELDS (RECEIVED)]"];
 
   if ([fetch count])
     {
@@ -1458,7 +1556,7 @@ static BOOL debugSoParts       = NO;
   NSString *value, *rc;
 
   rc = nil;
-  fetch = [self _fetchProperty: @"BODY[HEADER.FIELDS (REFERENCES)]"];
+  fetch = [self _fetchProperty: @"BODY.PEEK[HEADER.FIELDS (REFERENCES)]"];
 
   if ([fetch count])
     {
