@@ -1,6 +1,6 @@
 /* SOGoDomainDefaults.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2012 Inverse inc.
+ * Copyright (C) 2009-2014 Inverse inc.
  *
  * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
  *
@@ -176,6 +176,11 @@
   return [self stringArrayForKey: @"SOGoContactsDefaultRoles"];
 }
 
+- (NSString *) subscriptionFolderFormat
+{
+  return [self stringForKey: @"SOGoSubscriptionFolderFormat"];
+}
+
 //
 // In v2.0.4, SOGoForceIMAPLoginWithEmail was renamed to SOGoForceExternalLoginWithEmail
 // but we keep backward compatbility for now with previous versions.
