@@ -1,9 +1,7 @@
 /* SOGoGCSFolder.m - this file is part of SOGo
  *
  * Copyright (C) 2004-2005 SKYRIX Software AG
- * Copyright (C) 2006-2012 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2006-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -198,6 +196,7 @@ static NSArray *childRecordFields = nil;
       ocsPath = nil;
       ocsFolder = nil;
       childRecords = [NSMutableDictionary new];
+      folderSubscriptionValues = nil;
       userCanAccessAllObjects = NO;
     }
 
@@ -209,6 +208,7 @@ static NSArray *childRecordFields = nil;
   [ocsFolder release];
   [ocsPath release];
   [childRecords release];
+  [folderSubscriptionValues release];
   [super dealloc];
 }
 
@@ -277,29 +277,38 @@ static NSArray *childRecordFields = nil;
 {
   NSString *primaryDN;
   NSDictionary *ownerIdentity;
-
+  NSString *subjectFormat;
+  SOGoDomainDefaults *dd;
+  
   primaryDN = [row objectForKey: @"c_foldername"];
   if ([primaryDN length])
+  {
+    displayName = [NSMutableString new];
+    if ([primaryDN isEqualToString: [container defaultFolderName]])
+      [displayName appendString: [self labelForKey: primaryDN
+                                         inContext: context]];
+    else
+      [displayName appendString: primaryDN];
+    
+    if (!activeUserIsOwner)
     {
-      displayName = [NSMutableString new];
-      if ([primaryDN isEqualToString: [container defaultFolderName]])
-	[displayName appendString: [self labelForKey: primaryDN
-                                           inContext: context]];
-      else
-	[displayName appendString: primaryDN];
+      // We MUST NOT use SOGoUser instances here (by calling -primaryIdentity)
+      // as it'll load user defaults and user settings which is _very costly_
+      // since it involves JSON parsing and database requests
+      ownerIdentity = [[SOGoUserManager sharedUserManager]
+                       contactInfosForUserWithUIDorEmail: owner];
 
-      if (!activeUserIsOwner)
-	{
-	  // We MUST NOT use SOGoUser instances here (by calling -primaryIdentity)
-	  // as it'll load user defaults and user settings which is _very costly_
-	  // since it involves JSON parsing and database requests
-	  ownerIdentity = [[SOGoUserManager sharedUserManager]
-			    contactInfosForUserWithUIDorEmail: owner];
-
-	  [displayName appendFormat: @" (%@ <%@>)", [ownerIdentity objectForKey: @"cn"],
-		       [ownerIdentity objectForKey: @"c_email"]];
-	}
+      folderSubscriptionValues = [[NSMutableDictionary alloc] initWithObjectsAndKeys: displayName, @"FolderName",
+                                                                   [ownerIdentity objectForKey: @"cn"], @"UserName",
+                                                                   [ownerIdentity objectForKey: @"c_email"], @"Email", nil];
+      
+      dd = [[context activeUser] domainDefaults];
+      subjectFormat = [dd subscriptionFolderFormat];
+      
+      displayName = [folderSubscriptionValues keysWithFormat: subjectFormat];
+      [displayName retain];
     }
+  }
 }
 
 /* This method fetches the display name defined by the owner, but is also the
@@ -788,16 +797,22 @@ static NSArray *childRecordFields = nil;
 - (void) deleteEntriesWithIds: (NSArray *) ids
 {
   unsigned int count, max;
-  NSString *currentID;
+  NSEnumerator *names;
+  NSString *currentID, *currentName;
   SOGoContentObject *deleteObject;
 
   max = [ids count];
   for (count = 0; count < max; count++)
     {
       currentID = [ids objectAtIndex: count];
-      deleteObject = [self lookupName: currentID
+      names = [[currentID componentsSeparatedByString: @"/"] objectEnumerator];
+      deleteObject = self;
+      while ((currentName = [names nextObject]))
+        {
+          deleteObject = [deleteObject lookupName: currentName
 			   inContext: context
 			   acquire: NO];
+        }
       if (![deleteObject isKindOfClass: [NSException class]])
 	{
 	  if ([deleteObject respondsToSelector: @selector (prepareDelete)])
@@ -1093,8 +1108,15 @@ static NSArray *childRecordFields = nil;
   return @"";
 }
 
-- (NSArray *) _fetchSyncTokenFields: (NSDictionary *) properties
-                  matchingSyncToken: (NSString *) syncToken
+//
+// Method used to get all changes since a particular sync token
+//
+// It'll return standard properties (c_name, c_creationdate, etc...)
+// of new, modified and deleted components. 
+//
+- (NSArray *) syncTokenFieldsWithProperties: (NSDictionary *) properties
+                          matchingSyncToken: (NSString *) syncToken
+                                   fromDate: (NSCalendarDate *) theStartDate
 {
   /* TODO:
      - validation:
@@ -1118,8 +1140,20 @@ static NSArray *childRecordFields = nil;
   if ([syncToken length])
     {
       syncTokenInt = [syncToken intValue];
+      
       qualifier = [EOQualifier qualifierWithQualifierFormat:
                                  @"c_lastmodified > %d", syncTokenInt];
+
+      if (theStartDate)
+        {
+          EOQualifier *sinceDateQualifier = [EOQualifier qualifierWithQualifierFormat:
+                                                           @"c_creationdate > %d", (int)[theStartDate timeIntervalSince1970]];
+          
+          qualifier = [[EOAndQualifier alloc] initWithQualifiers: sinceDateQualifier, qualifier,
+                                              nil];
+          [qualifier autorelease];
+        }
+
       mRecords = [NSMutableArray arrayWithArray: [self _fetchFields: fields
                                                       withQualifier: qualifier
                                                       ignoreDeleted: YES]];
@@ -1139,10 +1173,24 @@ static NSArray *childRecordFields = nil;
         qualifier = [EOQualifier qualifierWithQualifierFormat: filter];
       else
         qualifier = nil;
-      records = [self _fetchFields: fields withQualifier: qualifier
+
+      if (theStartDate)
+        {
+          EOQualifier *sinceDateQualifier = [EOQualifier qualifierWithQualifierFormat:
+                                                           @"c_creationdate > %d", (int)[theStartDate timeIntervalSince1970]];
+          
+          qualifier = [[EOAndQualifier alloc] initWithQualifiers: sinceDateQualifier, qualifier,
+                                              nil];
+          [qualifier autorelease];
+        }
+      
+      records = [self _fetchFields: fields
+                     withQualifier: qualifier
                      ignoreDeleted: YES];
     }
 
+  
+
   return records;
 }
 
@@ -1319,11 +1367,15 @@ static NSArray *childRecordFields = nil;
     }
 
   NSZoneFree (NULL, selectors);
-
+  
+  /* If we haven't gotten any result to return, let's use the previously
+     supplied sync-token */
+  if (max == 0)
+    newToken = syncToken;
   /* If the most recent c_lastmodified is "now", we need to return "now - 1"
      in order to make sure during the next sync that every records that might
      get added at the same moment are not lost. */
-  if (!newToken || newToken == now)
+  else if (!newToken || newToken == now)
     newToken = now - 1;
 
   newTokenStr = [NSString stringWithFormat: @"%d", newToken];
@@ -1400,8 +1452,9 @@ static NSArray *childRecordFields = nil;
       propElement = [(NGDOMNodeWithChildren *) documentElement
                        firstElementWithTag: @"prop" inNamespace: XMLNS_WEBDAV];
       properties = [self parseDAVRequestedProperties: propElement];
-      records = [self _fetchSyncTokenFields: properties
-                          matchingSyncToken: syncToken];
+      records = [self syncTokenFieldsWithProperties: properties
+                                  matchingSyncToken: syncToken
+                                           fromDate: nil];
       [self _appendComponentProperties: [properties allKeys]
                            fromRecords: records
                      matchingSyncToken: [syncToken intValue]
