@@ -1,10 +1,6 @@
 /* UIxMailPartHTMLViewer.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2012 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
- *         Ludovic Marcotte <lmarcotte@inverse.ca>
- *         Francis Lachapelle <flachapelle@inverse.ca>
+ * Copyright (C) 2007-2013 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -546,7 +542,8 @@ static NSData* _sanitizeContent(NSData *theData)
                   else
                     skipAttribute = YES;
                 }
-              else if (([name isEqualToString: @"data"]
+              else if ([name isEqualToString: @"background"] ||
+                       ([name isEqualToString: @"data"]
                         || [name isEqualToString: @"classid"])
                        && [lowerName isEqualToString: @"object"])
                 {
@@ -562,6 +559,13 @@ static NSData* _sanitizeContent(NSData *theData)
                                    && ![value hasPrefix: @"mailto:"]
                                    && ![value hasPrefix: @"#"]);
                 }
+              // Avoid: <div style="background:url('http://www.sogo.nu/fileadmin/sogo/logos/sogo.bts.png' ); width: 200px; height: 200px;" title="ssss">
+              else if ([name isEqualToString: @"style"])
+                {
+                  value = [_attributes valueAtIndex: count];
+                  if ([value rangeOfString: @"url" options: NSCaseInsensitiveSearch].location != NSNotFound)
+                    name = [NSString stringWithFormat: @"unsafe-%@", name];
+                }
 	      else if (
 		       // Mouse Events
 		       [name isEqualToString: @"onclick"] ||
@@ -597,12 +601,13 @@ static NSData* _sanitizeContent(NSData *theData)
 		}
               else
                 value = [_attributes valueAtIndex: count];
+              
               if (!skipAttribute)
                 [resultPart appendFormat: @" %@=\"%@\"",
                             name, [value stringByReplacingString: @"\""
                                                       withString: @"\\\""]];
             }
-
+          
           if ([VoidTags containsObject: lowerName])
             [resultPart appendString: @"/"];
           [resultPart appendString: @">"];
@@ -689,16 +694,16 @@ static NSData* _sanitizeContent(NSData *theData)
         [self _appendStyle: _chars length: _len];
       else if (inBody)
         {
-	  NSString *tmpString;
+	  NSString *s;
   
-          tmpString = [NSString stringWithCharacters: _chars length: _len];
+          s = [NSString stringWithCharacters: _chars length: _len];
 
 	  // HACK: This is to avoid appending the useless junk in the <html> tag
 	  //       that Outlook adds. It seems to confuse the XML parser for
 	  //       forwarded messages as we get this in the _body_ of the email
 	  //       while we really aren't in it!
-	  if (![tmpString hasPrefix: @" xmlns:v=\"urn:schemas-microsoft-com:vml\""])
-	    [result appendString: [tmpString stringByEscapingHTMLString]];
+	  if (![s hasPrefix: @" xmlns:v=\"urn:schemas-microsoft-com:vml\""])
+	    [result appendString: [s stringByEscapingHTMLString]];
         }
     }
 }
@@ -843,7 +848,7 @@ static NSData* _sanitizeContent(NSData *theData)
              createXMLReaderForMimeType: @"text/html"];
 
   handler = [_UIxHTMLMailContentHandler new];
-  [handler setAttachmentIds: [mail fetchAttachmentIds]];
+  [handler setAttachmentIds: [mail fetchFileAttachmentIds]];
 
   // We check if we got an unsupported charset. If so
   // we convert everything to UTF-16{LE,BE} so it passes
@@ -951,7 +956,7 @@ static NSData* _sanitizeContent(NSData *theData)
     encoding = @"us-ascii";
 
   handler = [_UIxHTMLMailContentHandler new];
-  [handler setAttachmentIds: [mail fetchAttachmentIds]];
+  [handler setAttachmentIds: [mail fetchFileAttachmentIds]];
 
   // We check if we got an unsupported charset. If so
   // we convert everything to UTF-16{LE,BE} so it passes
