@@ -60,6 +60,7 @@
 #import <SOGo/SOGoUserSettings.h>
 
 #import <UI/Common/WODirectAction+SOGo.h>
+#import <UI/MailPartViewers/UIxMailSizeFormatter.h>
 
 #import "WOContext+UIxMailer.h"
 #import "UIxMailFormatter.h"
@@ -120,20 +121,9 @@
   return [dateFormatter formattedDateAndTime: messageDate];
 }
 
-- (NSString *) messageSize
+- (UIxMailSizeFormatter *) sizeFormatter
 {
-  NSString *rc;
-  int size;
-
-  size = [[message valueForKey: @"size"] intValue];
-  if (size > 1024*1024)
-    rc = [NSString stringWithFormat: @"%.1f MB", (float) size/1024/1024];
-  else if (size > 1024*100)
-    rc = [NSString stringWithFormat: @"%d KB", size/1024];    
-  else
-    rc = [NSString stringWithFormat: @"%.1f KB", (float) size/1024];
-  
-  return rc;
+  return [UIxMailSizeFormatter sharedMailSizeFormatter];
 }
 
 //
@@ -806,7 +796,7 @@
       [msg addObject: msgDate];
 
       // Size
-      [msg addObject: [self messageSize]];
+      [msg addObject: [[self sizeFormatter] stringForObjectValue: [message objectForKey: @"size"]]];
       
       // rowClasses
       [msg addObject: [self messageRowStyleClass]];
