@@ -1,6 +1,6 @@
 /* UIxPreferences.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WORequest.h>
 
+#import <NGImap4/NGSieveClient.h>
 #import <NGImap4/NSString+Imap4.h>
 
 #import <NGExtensions/NSObject+Logs.h>
@@ -41,13 +42,18 @@
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserDefaults.h>
 #import <SOGo/SOGoDomainDefaults.h>
+#import <SOGo/SOGoSieveManager.h>
 #import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoUserFolder.h>
+#import <SOGo/SOGoParentFolder.h>
 #import <SOGo/WOResourceManager+SOGo.h>
+#import <SOGo/SOGoBuild.h>
 #import <Mailer/SOGoMailAccount.h>
 #import <Mailer/SOGoMailAccounts.h>
 #import <Mailer/SOGoMailLabel.h>
 
+#import <Contacts/SOGoContactGCSFolder.h>
+
 #import "UIxPreferences.h"
 
 #warning this class is not finished
@@ -58,16 +64,62 @@
    workweek = from -> to
    identities */
 
+static NSArray *reminderItems = nil;
+static NSArray *reminderValues = nil;
+
 @implementation UIxPreferences
 
++ (void) initialize
+{
+  if (!reminderItems && !reminderValues)
+    {
+      reminderItems = [NSArray arrayWithObjects:
+			       @"5_MINUTES_BEFORE",
+			       @"10_MINUTES_BEFORE",
+			       @"15_MINUTES_BEFORE",
+			       @"30_MINUTES_BEFORE",
+			       @"45_MINUTES_BEFORE",
+			       @"-",
+			       @"1_HOUR_BEFORE",
+			       @"2_HOURS_BEFORE",
+			       @"5_HOURS_BEFORE",
+			       @"15_HOURS_BEFORE",
+			       @"-",
+			       @"1_DAY_BEFORE",
+			       @"2_DAYS_BEFORE",
+			       @"1_WEEK_BEFORE",
+			       nil];
+      reminderValues = [NSArray arrayWithObjects:
+				@"-PT5M",
+				@"-PT10M",
+				@"-PT15M",
+				@"-PT30M",
+				@"-PT45M",
+				@"",
+				@"-PT1H",
+				@"-PT2H",
+				@"-PT5H",
+				@"-PT15H",
+				@"",
+				@"-P1D",
+				@"-P2D",
+				@"-P1W",
+				nil];
+
+      [reminderItems retain];
+      [reminderValues retain];
+    }
+}
+
 - (id) init
 {
-  //NSDictionary *locale;
   SOGoDomainDefaults *dd;
-  
+
   if ((self = [super init]))
     {
       item = nil;
+      addressBooksIDWithDisplayName = nil;
+      client = nil;
 #warning user should be the owner rather than the activeUser
       ASSIGN (user, [context activeUser]);
       ASSIGN (today, [NSCalendarDate date]);
@@ -79,7 +131,7 @@
 
       label = nil;
       mailLabels = nil;
-      
+
       ASSIGN (daysOfWeek, [locale objectForKey: NSWeekDayNameArray]);
 
       dd = [user domainDefaults];
@@ -128,6 +180,8 @@
   [contactsCategories release];
   [forwardOptions release];
   [daysOfWeek release];
+  [addressBooksIDWithDisplayName release];
+  [client release];
   [super dealloc];
 }
 
@@ -173,7 +227,7 @@
       NSEnumerator *zones;
       BOOL found;
       unsigned int offset;
-      
+
       found = NO;
       now = [NSCalendarDate calendarDate];
       offset = [[userDefaults timeZone] secondsFromGMTForDate: now];
@@ -291,7 +345,7 @@
 
   if (![longDateFormatsList containsObject: [self userLongDateFormat]])
     [longDateFormatsList addObject: [self userLongDateFormat]];
-  
+
   return longDateFormatsList;
 }
 
@@ -317,7 +371,7 @@
 - (NSString *) userLongDateFormat
 {
   NSString *longDateFormat;
- 
+
   longDateFormat = [userDefaults longDateFormat];
   if (!longDateFormat)
     longDateFormat = @"default";
@@ -485,6 +539,57 @@
   return [userDefaults calendarTasksDefaultClassification];
 }
 
+- (NSArray *) reminderList
+{
+  return reminderItems;
+}
+
+- (NSString *) itemReminderText
+{
+  NSString *text;
+
+  if ([item isEqualToString: @"-"])
+    text = item;
+  else
+    text = [self labelForKey: [NSString stringWithFormat: @"reminder_%@", item]];
+
+  return text;
+}
+
+- (void) setReminder: (NSString *) theReminder
+{
+  NSString *value;
+  NSUInteger index;
+
+  index = NSNotFound;
+  value = @"NONE";
+
+  if (theReminder && [theReminder caseInsensitiveCompare: @"-"] != NSOrderedSame)
+    index = [reminderItems indexOfObject: theReminder];
+
+  if (index != NSNotFound)
+    value = [reminderValues objectAtIndex: index];
+
+  [userDefaults setCalendarDefaultReminder: value];
+}
+
+- (NSString *) reminder
+{
+  NSString *value;
+  NSUInteger index;
+
+  value = [userDefaults calendarDefaultReminder];
+  if (value != nil)
+    {
+      index = [reminderValues indexOfObject: value];
+
+      if (index != NSNotFound)
+        return [reminderItems objectAtIndex: index];
+    }
+
+  return @"NONE";
+}
+
 - (NSArray *) hoursList
 {
   static NSMutableArray *hours = nil;
@@ -558,59 +663,17 @@
   [userDefaults setFirstWeekOfYear: newFirstWeek];
 }
 
-- (BOOL) reminderEnabled
-{
-  return [userDefaults reminderEnabled];
-}
-
-- (void) setReminderEnabled: (BOOL) newValue
-{
-  [userDefaults setReminderEnabled: newValue];
-}
-
-- (BOOL) remindWithASound
-{
-  return [userDefaults remindWithASound];
-}
-
-- (void) setRemindWithASound: (BOOL) newValue
-{
-  [userDefaults setRemindWithASound: newValue];
-}
-
-- (NSArray *) reminderTimesList
-{
-  static NSArray *reminderTimesList = nil;
-
-  if (!reminderTimesList)
-    {
-      reminderTimesList = [NSArray arrayWithObjects: @"0000", @"0005",
-                                   @"0010", @"0015", @"0030", @"0100",
-                                   @"0200", @"0400", @"0800", @"1200",
-                                   @"2400", @"4800", nil];
-      [reminderTimesList retain];
-    }
-
-  return reminderTimesList;
-}
-
-- (NSString *) itemReminderTimeText
-{
-  return [self labelForKey:
-                 [NSString stringWithFormat: @"reminderTime_%@", item]];
-}
-
-- (NSString *) userReminderTime
-{
-  return [userDefaults reminderTime];
-}
-
-- (void) setReminderTime: (NSString *) newTime
-{
-  [userDefaults setReminderTime: newTime];
-}
-
 /* Mailer */
+- (void) setAddOutgoingAddresses: (BOOL) addOutgoingAddresses
+{
+  [userDefaults setMailAddOutgoingAddresses: addOutgoingAddresses];
+}
+
+- (BOOL) addOutgoingAddresses
+{
+  return [userDefaults mailAddOutgoingAddresses];
+}
+
 - (void) setShowSubscribedFoldersOnly: (BOOL) showSubscribedFoldersOnly
 {
   [userDefaults setMailShowSubscribedFoldersOnly: showSubscribedFoldersOnly];
@@ -631,6 +694,69 @@
   return [userDefaults mailSortByThreads];
 }
 
+- (NSArray *) addressBookList
+{
+  /* We want all the SourceIDS */
+  NSMutableArray *folders, *availableAddressBooksID, *availableAddressBooksName;
+  SOGoParentFolder *contactFolders;
+
+  int i, count;
+  BOOL collectedAlreadyExist;
+
+  contactFolders = [[[context activeUser] homeFolderInContext: context]
+                    lookupName: @"Contacts"
+                    inContext: context
+                    acquire: NO];
+  folders = [NSMutableArray arrayWithArray: [contactFolders subFolders]];
+  count = [folders count]-1;
+
+  // Inside this loop we remove all the public or shared addressbooks
+  for (; count >= 0; count--)
+  {
+    if (![[folders objectAtIndex: count] isKindOfClass: [SOGoContactGCSFolder class]])
+      [folders removeObjectAtIndex: count];
+  }
+
+  // Parse the objects in order to have only the displayName of the addressbooks to be displayed on the preferences interface
+  availableAddressBooksID = [NSMutableArray arrayWithCapacity: [folders count]];
+  availableAddressBooksName = [NSMutableArray arrayWithCapacity: [folders count]];
+  count = [folders count]-1;
+  collectedAlreadyExist = NO;
+
+  for (i = 0; i <= count ; i++) {
+    [availableAddressBooksID addObject:[[folders objectAtIndex:i] realNameInContainer]];
+    [availableAddressBooksName addObject:[[folders objectAtIndex:i] displayName]];
+
+    if ([[availableAddressBooksID objectAtIndex:i] isEqualToString: @"collected"])
+      collectedAlreadyExist = YES;
+  }
+  // Create the dictionary for the next function : itemAddressBookText.
+  if (!addressBooksIDWithDisplayName)
+    addressBooksIDWithDisplayName = [[NSMutableDictionary alloc] initWithObjects:availableAddressBooksName
+                                                                         forKeys:availableAddressBooksID];
+    if (!collectedAlreadyExist)
+    {
+      [availableAddressBooksID addObject: @"collected"];
+      [addressBooksIDWithDisplayName setObject: [self labelForKey: @"Collected Address Book"] forKey: @"collected"];
+    }
+
+  return availableAddressBooksID;
+}
+- (NSString *) itemAddressBookText
+{
+  return [addressBooksIDWithDisplayName objectForKey: item];
+}
+
+- (NSString *) userAddressBook
+{
+  return [userDefaults selectedAddressBook];
+}
+
+- (void) setUserAddressBook: (NSString *) newSelectedAddressBook
+{
+  [userDefaults setSelectedAddressBook: newSelectedAddressBook];
+}
+
 - (NSArray *) messageCheckList
 {
   NSArray *intervalsList;
@@ -795,16 +921,15 @@
 
 - (NSString *) sieveCapabilities
 {
-#warning sieve caps should be deduced from the server
   static NSArray *capabilities = nil;
 
   if (!capabilities)
     {
-      capabilities = [NSArray arrayWithObjects: @"fileinto", @"reject",
-                              @"envelope", @"vacation", @"imapflags",
-                              @"notify", @"subaddress", @"relational",
-                              @"comparator-i;ascii-numeric", @"regex", nil];
-      [capabilities retain];
+      if ([self sieveClient])
+        capabilities = [[self sieveClient] capabilities];
+      else
+        capabilities = [NSArray array];
+        [capabilities retain];
     }
 
   return [capabilities jsonRepresentation];
@@ -885,7 +1010,7 @@
 - (NSString *) autoReplyEmailAddresses
 {
   NSArray *addressesList;
- 
+
   addressesList = [vacationOptions objectForKey: @"autoReplyEmailAddresses"];
 
   return (addressesList
@@ -938,7 +1063,7 @@
   obj = [vacationOptions objectForKey: @"ignoreLists"];
 
   if (obj == nil)
-    ignore = YES; // defaults to true
+    ignore = YES; // defaults to YES
   else
     ignore = [obj boolValue];
 
@@ -964,7 +1089,7 @@
 - (void) setVacationEndDate: (NSCalendarDate *) endDate
 {
   NSNumber *time;
-  
+
   time = [NSNumber numberWithInt: [endDate timeIntervalSince1970]];
 
   [vacationOptions setObject: time forKey: @"endDate"];
@@ -1082,19 +1207,47 @@
     }
 }
 
+- (NSString *) sogoVersion
+{
+  // The variable SOGoVersion comes from the import: SOGo/Build.h
+  return [NSString stringWithString: SOGoVersion];
+}
+
+- (id) sieveClient
+{
+  SOGoMailAccount *account;
+  SOGoMailAccounts *folder;
+  SOGoSieveManager *manager;
+
+  if (!client)
+  {
+    folder = [[self clientObject] mailAccountsFolder: @"Mail" inContext: context];
+    account = [folder lookupName: @"0" inContext: context acquire: NO];
+    manager = [SOGoSieveManager sieveManagerForUser: [context activeUser]];
+    client = [[manager clientForAccount: account] retain];
+  }
+
+  return client;
+}
+
+- (BOOL) isSieveServerAvailable
+{
+  return (([(NGSieveClient *)[self sieveClient] isConnected])
+          ? YES
+          : NO);
+}
+
 - (id <WOActionResults>) defaultAction
 {
   id <WOActionResults> results;
-  WORequest *request;
   SOGoDomainDefaults *dd;
-  NSString *method;
+  SOGoMailAccount *account;
+  SOGoMailAccounts *folder;
+  WORequest *request;
 
   request = [context request];
   if ([[request method] isEqualToString: @"POST"])
     {
-      SOGoMailAccount *account;
-      SOGoMailAccounts *folder;
-
       dd = [[context activeUser] domainDefaults];
       if ([dd sieveScriptsEnabled])
         [userDefaults setSieveFilters: sieveFilters];
@@ -1103,19 +1256,26 @@
       if ([dd forwardEnabled])
         [userDefaults setForwardOptions: forwardOptions];
 
-      [userDefaults synchronize];
+      if (!([dd sieveScriptsEnabled] || [dd vacationEnabled] || [dd forwardEnabled]) || [self isSieveServerAvailable])
+        {
+          [userDefaults synchronize];
+          folder = [[self clientObject] mailAccountsFolder: @"Mail"
+                                                 inContext: context];
+          account = [folder lookupName: @"0" inContext: context acquire: NO];
 
-      folder = [[self clientObject] mailAccountsFolder: @"Mail"
-                                             inContext: context];
-      account = [folder lookupName: @"0" inContext: context acquire: NO];
-      [account updateFilters];
+          if ([account updateFilters])
+            // If Sieve is not enabled, the SOGoSieveManager will immediatly return a positive answer
+            // See [SOGoSieveManager updateFiltersForAccount:withUsername:andPassword:]
+            results = [self responseWithStatus: 200
+                         andJSONRepresentation: [NSDictionary dictionaryWithObjectsAndKeys: [NSNumber numberWithBool:hasChanged], @"hasChanged", nil]];
 
-      if (hasChanged)
-        method = @"window.location.reload()";
-      else
-        method = nil;
-
-      results = [self jsCloseWithRefreshMethod: method];
+          else
+            results = [self responseWithStatus: 502
+                         andJSONRepresentation: [NSDictionary dictionaryWithObjectsAndKeys: @"Connection error", @"textStatus", nil]];
+        }
+    else
+      results = [self responseWithStatus: 503
+                   andJSONRepresentation: [NSDictionary dictionaryWithObjectsAndKeys: @"Service temporarily unavailable", @"textStatus", nil]];
     }
   else
     results = self;
@@ -1195,11 +1355,11 @@
   if (!mailLabels)
     {
       NSDictionary *v;
-     
-      v = [[[context activeUser] userDefaults] mailLabelsColors];      
+
+      v = [[[context activeUser] userDefaults] mailLabelsColors];
       ASSIGN(mailLabels, [SOGoMailLabel labelsFromDefaults: v  component: self]);
     }
-  
+
   return mailLabels;
 }
 
@@ -1298,7 +1458,7 @@
                        componentsSeparatedByString: @","];
   if (!categoryLabels)
     categoryLabels = [NSArray array];
-  
+
   return [categoryLabels trimmedComponents];
 }
 
@@ -1414,7 +1574,7 @@
       action = [receipts objectForKey: @"receiptAction"];
       [userDefaults
         setAllowUserReceipt: [action isEqualToString: @"allow"]];
-      
+
       action = [receipts objectForKey: @"receiptNonRecipientAction"];
       if ([self _validateReceiptAction: action])
         [userDefaults setUserReceiptNonRecipientAction: action];
@@ -1422,7 +1582,7 @@
       action = [receipts objectForKey: @"receiptOutsideDomainAction"];
       if ([self _validateReceiptAction: action])
         [userDefaults setUserReceiptOutsideDomainAction: action];
-      
+
       action = [receipts objectForKey: @"receiptAnyAction"];
       if ([self _validateReceiptAction: action])
         [userDefaults setUserReceiptAnyAction: action];
