@@ -1007,9 +1007,7 @@ SOGoEventDragController.prototype = {
     onDragStart: function SEDC_onDragStart(event) {
         var target = getTarget(event);
         if (eventIsLeftClick(event) && (target.nodeType == 1)) {
-            if (target.hasClassName("minutes15") ||
-                target.hasClassName("minutes30") ||
-                target.hasClassName("minutes45"))
+            if (/minutes\d{2}/.test(target.className))
                 target = target.parentNode;
             if ((!this.eventCells
                  && (target.hasClassName("clickableHourCell")
@@ -1059,11 +1057,8 @@ SOGoEventDragController.prototype = {
                     Event.observe(window, "mouseup", this.onDragStopBound);
                 this.onDragModeBound = this.onDragMode.bindAsEventListener(this);
                 Event.observe(document.body, "mousemove", this.onDragModeBound);
-                Event.stop(event);
             }
         }
-
-        return false;
     },
 
     _determineEventInvitation: function SEDC__determineEventType(node) {
